package consul

import (
	"log"
	"strconv"
	"time"

	consul "github.com/hashicorp/consul/api"
)

type Service struct {
	Name        string
	Address     string
	Port        int
	TTL         time.Duration
	ConsulAgent *consul.Agent
}

func NewService(appName, appDomain string, port int) (*Service, error) {
	s := new(Service)
	s.Name = appName
	s.Address = appDomain
	s.Port = port
	s.TTL = time.Second * 15

	client, err := consul.NewClient(newClientConfig())
	if err != nil {
		return nil, err
	}
	s.ConsulAgent = client.Agent()

	// tags := s.GetTags()

	return s, nil
}

func newClientConfig() *consul.Config {
	conf := consul.DefaultConfig()
	conf.Address = "api-registry:8500" // FIXME:consul server

	return conf
}

func (s *Service) Register() error {
	serviceDef := &consul.AgentServiceRegistration{
		Name:    s.Name,
		Address: s.Address,
		Port:    s.Port,
		Tags:    s.getTags(),
		Check: &consul.AgentServiceCheck{
			TTL: s.TTL.String(),
		},
	}

	if err := s.ConsulAgent.ServiceRegister(serviceDef); err != nil {
		return err
	}
	go s.UpdateTTL(serviceDef /*service.Check*/)

	return nil
}

func (s *Service) UpdateTTL(service *consul.AgentServiceRegistration /*check func() (bool, error)*/) {
	ticker := time.NewTicker(s.TTL / 2)
	for range ticker.C {
		s.update(service /*check*/)
	}
}

func (s *Service) update(service *consul.AgentServiceRegistration /*check func() (bool, error)*/) {
	// ok, err := check()
	// if !ok {
	// 	log.Printf("err=\"Check failed\" msg=\"%s\"", err.Error())
	// 	if err := s.ConsulAgent.FailTTL("service:"+s.Name, err.Error()); err != nil {
	// 		log.Println(err)
	// 	}
	// } else {
	// log.Println("Updating service info...")
	if err := s.ConsulAgent.PassTTL("service:"+service.Name, ""); err != nil {
		log.Println(err)
	}
	// }
}

func (s *Service) getTags() []string {
	name, addr, port := s.Name, s.Address, strconv.Itoa(s.Port)

	tags := []string{
		"traefik.enable=true",
		"traefik.http.routers." + name + ".rule=Host(`" + addr + "`)",
		"traefik.http.routers." + name + ".entryPoints=https",
		"traefik.http.routers." + name + ".service=" + name,
		"traefik.http.routers." + name + ".middlewares=compress,requestid",
		"traefik.http.routers." + name + ".tls=true",
		"traefik.http.services." + name + ".loadbalancer.server.scheme=http",
		"traefik.http.services." + name + ".loadbalancer.server.port=" + port,
		"traefik.http.services." + name + ".loadbalancer.passhostheader=false",
		"traefik.http.middlewares.compress.compress=true",
		"traefik.http.middlewares.requestid.plugin.requestid.headerName=X-Request-ID",
		// "traefik.http.services." + name + ".loadbalancer.healthcheck.path=/health",
		// "traefik.http.services." + name + ".loadbalancer.healthcheck.interval=10s",
		// "traefik.tls.certificates.certfile=/certs/client.cert",
		// "traefik.tls.certificates.keyfile=/certs/client.key",
	}

	return tags
}
